"""Initial migration

Revision ID: 1
Revises:
Create Date: 2017-12-29 14:39:27.854291

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '1'
down_revision = None


def upgrade():
    pass


def downgrade():
    pass
