# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: server_certificate_v1.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='server_certificate_v1.proto',
  package='app.protobufs.v1',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1bserver_certificate_v1.proto\x12\x10\x61pp.protobufs.v1\"J\n\x1eSavedServerCertificateResponse\x12\x13\n\x0b\x63\x65rtificate\x18\x01 \x01(\t\x12\x13\n\x0bprivate_key\x18\x02 \x01(\tb\x06proto3'
)




_SAVEDSERVERCERTIFICATERESPONSE = _descriptor.Descriptor(
  name='SavedServerCertificateResponse',
  full_name='app.protobufs.v1.SavedServerCertificateResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='certificate', full_name='app.protobufs.v1.SavedServerCertificateResponse.certificate', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='private_key', full_name='app.protobufs.v1.SavedServerCertificateResponse.private_key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=49,
  serialized_end=123,
)

DESCRIPTOR.message_types_by_name['SavedServerCertificateResponse'] = _SAVEDSERVERCERTIFICATERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SavedServerCertificateResponse = _reflection.GeneratedProtocolMessageType('SavedServerCertificateResponse', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDSERVERCERTIFICATERESPONSE,
  '__module__' : 'server_certificate_v1_pb2'
  # @@protoc_insertion_point(class_scope:app.protobufs.v1.SavedServerCertificateResponse)
  })
_sym_db.RegisterMessage(SavedServerCertificateResponse)


# @@protoc_insertion_point(module_scope)
