"""Initial migration

Revision ID: 5
Revises:
Create Date: 2017-12-29 14:39:27.854291

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '5'
down_revision = '4'


def upgrade():
    pass


def downgrade():
    pass
